# --------------------------
# -------- OVERVIEW --------
# --------------------------
#
# This script forms the basis of a Data Input Method for Cacti.
# The purpose is to query a Cisco Wireless Access Point for all
# connected clients' current TX rate
# These values are indexed against client MAC addresses and are
# not directly graphable. So we return a count of the number of
# clients within particular 'rates'
# This should provide a more precise indication of the general
# service offered by an AP than does the Average Client RSSI
#
# Note: The data is returned in a format designed for a Cacti AREA/STACK graph
#
# ---------------------------
# -------- CHANGELOG --------
# ---------------------------
#
# 20081003 (BC)
#       Initial version
# 20081007 (BC)
#       Changed rate_avg to output an integer, rrdtool was storing NaN's
# 20081016 (BC)
#       Added debug mode that prints error messages - normally suppressed by returning '0' for all values
#
# ----------------------------------------
# -------- CAVEATS / BUGS / TO DO --------
# ----------------------------------------
#
# -----------------------------
# -------- GLOBAL VARS --------
# -----------------------------

set version 1.1

# Default vars
set oid .1.3.6.1.4.1.9.9.273.1.3.1.1.1
set debug 0
set usage "Usage: $argv0 <?-d?> <hostname|ip_address> <community_string> <snmp_version>"
set result_err "rate_1_cnt:0 rate_2_cnt:0 rate_5_5_cnt:0 rate_6_cnt:0 rate_9_cnt:0 rate_11_cnt:0 rate_12_cnt:0 rate_18_cnt:0 rate_24_cnt:0 rate_36_cnt:0 rate_48_cnt:0 rate_54_cnt:0 rate_avg:0"
set valid_rates [list 1.0 2.0 5.5 6.0 9.0 11.0 12.0 18.0 24.0 36.0 48.0 54.0]

#set comstr public
#set ver 2c
#set host 10.10.6.146

# ------------------------------
# -------- PROCESS ARGS --------
# ------------------------------

# Process command line options
while { [string match -* [set option [lindex $argv 0]]] == 1 } {

        switch -- $option {
		-d {
			set argv [lreplace $argv 0 0]
			set debug 1
			break
		}
		-v {
			puts "Script version: $version"
			exit
		}
                default {
                        break
                }
	}
}

if {[llength $argv] != 3} {
	if { $debug } { puts stderr "Error: You didn't provide three arguments to the script"
	} else { puts $result_err
	}
	exit 1
}

set host [lindex $argv 0]
set comstr [lindex $argv 1]
set ver [lindex $argv 2] ; if { $ver eq 2 } { set ver 2c }

#puts "snmpwalk -Ox -v $ver -c $comstr $host $oid"

# ------------------------------
# -------- START SCRIPT --------
# ------------------------------

if { [catch { set walk_result [exec snmpwalk -Ox -v $ver -c $comstr $host $oid] } result] } {
	if { $debug } { puts stderr "Error: There was a problem running snmpwalk:\n\t$result"
	} else { puts $result_err
	}
	exit 1	

} else {
	set rates ""
	foreach line [split $walk_result "\n"] {
		if { [regexp { = Hex-STRING: ([0-9A-Fa-f]{2}) $} $line xxx hrate] } {
			# Convert from hex and adjust to Mbps
			set rate [expr [scan $hrate %2x] * 0.5]
			# Sanity check rate
			if { [lsearch $valid_rates $rate] == -1 } {
				if { $debug } { puts stderr "Error: Invalid client rate returned: $rate"
				} else { puts $result_err
				}
				exit 1
			}
			#puts "hrate is $hrate, rate is $rate"
			lappend rates $rate
		} else {
			if { $debug } { puts stderr "Error: No valid result received from snmpwalk: $line"
			} else { puts $result_err
			}
			exit 1
		}
	}
	#puts "rates is $rates"
	set rate_cnt [llength $rates]
	#puts "rate_cnt is $rate_cnt"

	# Initialise counters for each rate
	set rate_1_cnt 0; set rate_2_cnt 0; set rate_5_5_cnt 0; set rate_6_cnt 0; set rate_9_cnt 0; set rate_11_cnt 0;
	set rate_12_cnt 0; set rate_18_cnt 0; set rate_24_cnt 0; set rate_36_cnt 0; set rate_48_cnt 0; set rate_54_cnt 0;
	set rate_sum 0

	# Loop through the values, incrementing counters for each rate
	foreach rate $rates {
		switch -- $rate {
			1.0 { incr rate_1_cnt }
			2.0 { incr rate_2_cnt }
			5.5 { incr rate_5_5_cnt }
			6.0 { incr rate_6_cnt }
			9.0 { incr rate_9_cnt }
			11.0 { incr rate_11_cnt }
			12.0 { incr rate_12_cnt }
			18.0 { incr rate_18_cnt }
			24.0 { incr rate_24_cnt }
			36.0 { incr rate_36_cnt }
			48.0 { incr rate_48_cnt }
			54.0 { incr rate_54_cnt }
		}
		set rate_sum [expr $rate_sum + $rate]
	}

	# Calculate the average for this poll - may be used to plot a moving average
	#puts "rate_sum is $rate_sum"
	set rate_avg [format %.0f [expr $rate_sum / $rate_cnt]]
	#puts "rate_avg is $rate_avg"

	# Construct the output string
	set result_str "rate_1_cnt:$rate_1_cnt rate_2_cnt:$rate_2_cnt rate_5_5_cnt:$rate_5_5_cnt rate_6_cnt:$rate_6_cnt rate_9_cnt:$rate_9_cnt rate_11_cnt:$rate_11_cnt rate_12_cnt:$rate_12_cnt rate_18_cnt:$rate_18_cnt rate_24_cnt:$rate_24_cnt rate_36_cnt:$rate_36_cnt rate_48_cnt:$rate_48_cnt rate_54_cnt:$rate_54_cnt rate_avg:$rate_avg"
	puts $result_str
	#return $result_str
}
